@@ -1,4 +1,4 @@
-// Copyright 2020-2024 Buf Technologies, Inc.
+// Copyright 2020-2025 Buf Technologies, Inc.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -29,16 +29,18 @@ import (
 	"github.com/bufbuild/buf/private/buf/bufwkt/bufwktstore"
 	"github.com/bufbuild/buf/private/buf/bufworkspace"
 	"github.com/bufbuild/buf/private/bufpkg/bufanalysis"
+	"github.com/bufbuild/buf/private/bufpkg/bufcheck"
 	"github.com/bufbuild/buf/private/bufpkg/bufconfig"
 	"github.com/bufbuild/buf/private/bufpkg/bufimage"
 	"github.com/bufbuild/buf/private/bufpkg/bufimage/bufimageutil"
 	"github.com/bufbuild/buf/private/bufpkg/bufmodule"
+	"github.com/bufbuild/buf/private/bufpkg/bufparse"
+	"github.com/bufbuild/buf/private/bufpkg/bufplugin"
 	"github.com/bufbuild/buf/private/bufpkg/bufreflect"
 	"github.com/bufbuild/buf/private/gen/data/datawkt"
 	imagev1 "github.com/bufbuild/buf/private/gen/proto/go/buf/alpha/image/v1"
 	"github.com/bufbuild/buf/private/pkg/app"
 	"github.com/bufbuild/buf/private/pkg/app/appcmd"
-	"github.com/bufbuild/buf/private/pkg/command"
 	"github.com/bufbuild/buf/private/pkg/git"
 	"github.com/bufbuild/buf/private/pkg/httpauth"
 	"github.com/bufbuild/buf/private/pkg/ioext"
@@ -47,8 +49,8 @@ import (
 	"github.com/bufbuild/buf/private/pkg/slicesext"
 	"github.com/bufbuild/buf/private/pkg/storage/storageos"
 	"github.com/bufbuild/buf/private/pkg/syserror"
+	"github.com/bufbuild/buf/private/pkg/wasm"
 	"github.com/bufbuild/protovalidate-go"
-	"go.uber.org/multierr"
 	"google.golang.org/protobuf/proto"
 )
 
@@ -89,11 +91,19 @@ type Controller interface {
 		workspace bufworkspace.Workspace,
 		options ...FunctionOption,
 	) (bufimage.Image, error)
-	GetTargetImageWithConfigs(
+	// GetTargetImageWithConfigsAndCheckClient gets the target ImageWithConfigs
+	// with a configured bufcheck Client.
+	//
+	// ImageWithConfig scopes the configuration per image for use with breaking
+	// and lint checks. The check Client is bound to the input to ensure that the
+	// correct remote plugin dependencies are used. A wasmRuntime is provided
+	// to evaluate Wasm plugins.
+	GetTargetImageWithConfigsAndCheckClient(
 		ctx context.Context,
 		input string,
+		wasmRuntime wasm.Runtime,
 		options ...FunctionOption,
-	) ([]ImageWithConfig, error)
+	) ([]ImageWithConfig, bufcheck.Client, error)
 	// GetImportableImageFileInfos gets the importable .proto FileInfos for the given input.
 	//
 	// This includes all files that can be possible imported. For example, if a Module
@@ -128,6 +138,16 @@ type Controller interface {
 		defaultMessageEncoding buffetch.MessageEncoding,
 		options ...FunctionOption,
 	) error
+	// GetCheckClientForWorkspace returns a new bufcheck Client for the given Workspace.
+	//
+	// Clients are bound to a specific Workspace to ensure that the correct
+	// plugin dependencies are used. A wasmRuntime is provided to evaluate
+	// Wasm plugins.
+	GetCheckClientForWorkspace(
+		ctx context.Context,
+		workspace bufworkspace.Workspace,
+		wasmRuntime wasm.Runtime,
+	) (bufcheck.Client, error)
 }
 
 func NewController(
@@ -137,6 +157,8 @@ func NewController(
 	moduleKeyProvider bufmodule.ModuleKeyProvider,
 	moduleDataProvider bufmodule.ModuleDataProvider,
 	commitProvider bufmodule.CommitProvider,
+	pluginKeyProvider bufplugin.PluginKeyProvider,
+	pluginDataProvider bufplugin.PluginDataProvider,
 	wktStore bufwktstore.Store,
 	httpClient *http.Client,
 	httpauthAuthenticator httpauth.Authenticator,
@@ -150,6 +172,8 @@ func NewController(
 		moduleKeyProvider,
 		moduleDataProvider,
 		commitProvider,
+		pluginKeyProvider,
+		pluginDataProvider,
 		wktStore,
 		httpClient,
 		httpauthAuthenticator,
@@ -171,6 +195,8 @@ type controller struct {
 	moduleDataProvider bufmodule.ModuleDataProvider
 	graphProvider      bufmodule.GraphProvider
 	commitProvider     bufmodule.CommitProvider
+	pluginKeyProvider  bufplugin.PluginKeyProvider
+	pluginDataProvider bufplugin.PluginDataProvider
 	wktStore           bufwktstore.Store
 
 	disableSymlinks           bool
@@ -178,7 +204,6 @@ type controller struct {
 	fileAnnotationsToStdout   bool
 	copyToInMemory            bool
 
-	commandRunner               command.Runner
 	storageosProvider           storageos.Provider
 	buffetchRefParser           buffetch.RefParser
 	buffetchReader              buffetch.Reader
@@ -194,6 +219,8 @@ func newController(
 	moduleKeyProvider bufmodule.ModuleKeyProvider,
 	moduleDataProvider bufmodule.ModuleDataProvider,
 	commitProvider bufmodule.CommitProvider,
+	pluginKeyProvider bufplugin.PluginKeyProvider,
+	pluginDataProvider bufplugin.PluginDataProvider,
 	wktStore bufwktstore.Store,
 	httpClient *http.Client,
 	httpauthAuthenticator httpauth.Authenticator,
@@ -206,6 +233,8 @@ func newController(
 		graphProvider:      graphProvider,
 		moduleDataProvider: moduleDataProvider,
 		commitProvider:     commitProvider,
+		pluginKeyProvider:  pluginKeyProvider,
+		pluginDataProvider: pluginDataProvider,
 		wktStore:           wktStore,
 	}
 	for _, option := range options {
@@ -214,7 +243,6 @@ func newController(
 	if err := validateFileAnnotationErrorFormat(controller.fileAnnotationErrorFormat); err != nil {
 		return nil, err
 	}
-	controller.commandRunner = command.NewRunner()
 	controller.storageosProvider = newStorageosProvider(controller.disableSymlinks)
 	controller.buffetchRefParser = buffetch.NewRefParser(logger)
 	controller.buffetchReader = buffetch.NewReader(
@@ -225,7 +253,6 @@ func newController(
 		git.NewCloner(
 			logger,
 			controller.storageosProvider,
-			controller.commandRunner,
 			gitClonerOptions,
 		),
 		moduleKeyProvider,
@@ -236,6 +263,7 @@ func newController(
 		graphProvider,
 		moduleDataProvider,
 		commitProvider,
+		pluginKeyProvider,
 	)
 	controller.workspaceDepManagerProvider = bufworkspace.NewWorkspaceDepManagerProvider(
 		logger,
@@ -326,11 +354,12 @@ func (c *controller) GetImageForWorkspace(
 	return c.getImageForWorkspace(ctx, workspace, functionOptions)
 }
 
-func (c *controller) GetTargetImageWithConfigs(
+func (c *controller) GetTargetImageWithConfigsAndCheckClient(
 	ctx context.Context,
 	input string,
+	wasmRuntime wasm.Runtime,
 	options ...FunctionOption,
-) (_ []ImageWithConfig, retErr error) {
+) (_ []ImageWithConfig, _ bufcheck.Client, retErr error) {
 	defer c.handleFileAnnotationSetRetError(&retErr)
 	functionOptions := newFunctionOptions(c)
 	for _, option := range options {
@@ -338,42 +367,41 @@ func (c *controller) GetTargetImageWithConfigs(
 	}
 	ref, err := c.buffetchRefParser.GetRef(ctx, input)
 	if err != nil {
-		return nil, err
+		return nil, nil, err
 	}
+	var workspace bufworkspace.Workspace
 	switch t := ref.(type) {
 	case buffetch.ProtoFileRef:
-		workspace, err := c.getWorkspaceForProtoFileRef(ctx, t, functionOptions)
+		workspace, err = c.getWorkspaceForProtoFileRef(ctx, t, functionOptions)
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
-		return c.buildTargetImageWithConfigs(ctx, workspace, functionOptions)
 	case buffetch.SourceRef:
-		workspace, err := c.getWorkspaceForSourceRef(ctx, t, functionOptions)
+		workspace, err = c.getWorkspaceForSourceRef(ctx, t, functionOptions)
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
-		return c.buildTargetImageWithConfigs(ctx, workspace, functionOptions)
 	case buffetch.ModuleRef:
-		workspace, err := c.getWorkspaceForModuleRef(ctx, t, functionOptions)
+		workspace, err = c.getWorkspaceForModuleRef(ctx, t, functionOptions)
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
-		return c.buildTargetImageWithConfigs(ctx, workspace, functionOptions)
 	case buffetch.MessageRef:
 		image, err := c.getImageForMessageRef(ctx, t, functionOptions)
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
 		bucket, err := c.storageosProvider.NewReadWriteBucket(
 			".",
 			storageos.ReadWriteBucketWithSymlinksIfSupported(),
 		)
 		if err != nil {
-			return nil, err
+			return nil, nil, err
 		}
 		lintConfig := bufconfig.DefaultLintConfigV1
 		breakingConfig := bufconfig.DefaultBreakingConfigV1
 		var pluginConfigs []bufconfig.PluginConfig
+		pluginKeyProvider := bufplugin.NopPluginKeyProvider
 		bufYAMLFile, err := bufconfig.GetBufYAMLFileForPrefixOrOverride(
 			ctx,
 			bucket,
@@ -382,16 +410,15 @@ func (c *controller) GetTargetImageWithConfigs(
 		)
 		if err != nil {
 			if !errors.Is(err, fs.ErrNotExist) {
-				return nil, err
+				return nil, nil, err
 			}
 			// We did not find a buf.yaml in our current directory, and there was no config override.
 			// Use the defaults.
 		} else {
-			pluginConfigs = bufYAMLFile.PluginConfigs()
 			if topLevelLintConfig := bufYAMLFile.TopLevelLintConfig(); topLevelLintConfig == nil {
 				// Ensure that this is a v2 config
 				if fileVersion := bufYAMLFile.FileVersion(); fileVersion != bufconfig.FileVersionV2 {
-					return nil, syserror.Newf("non-v2 version with no top-level lint config: %s", fileVersion)
+					return nil, nil, syserror.Newf("non-v2 version with no top-level lint config: %s", fileVersion)
 				}
 				// v2 config without a top-level lint config, use v2 default
 				lintConfig = bufconfig.DefaultLintConfigV2
@@ -400,26 +427,87 @@ func (c *controller) GetTargetImageWithConfigs(
 			}
 			if topLevelBreakingConfig := bufYAMLFile.TopLevelBreakingConfig(); topLevelBreakingConfig == nil {
 				if fileVersion := bufYAMLFile.FileVersion(); fileVersion != bufconfig.FileVersionV2 {
-					return nil, syserror.Newf("non-v2 version with no top-level breaking config: %s", fileVersion)
+					return nil, nil, syserror.Newf("non-v2 version with no top-level breaking config: %s", fileVersion)
 				}
 				// v2 config without a top-level breaking config, use v2 default
 				breakingConfig = bufconfig.DefaultBreakingConfigV2
 			} else {
 				breakingConfig = topLevelBreakingConfig
 			}
+			// The directory path is resolved to a buf.yaml file and a buf.lock file. If the
+			// buf.yaml file is found, the PluginConfigs from the buf.yaml file and the PluginKeys
+			// from the buf.lock file are resolved to create the PluginKeyProvider.
+			pluginConfigs = bufYAMLFile.PluginConfigs()
+			// If a config override is provided, the PluginConfig remote Refs use the BSR
+			// to resolve the PluginKeys. No buf.lock is required.
+			// If the buf.yaml file is not found, the bufplugin.NopPluginKeyProvider is returned.
+			// If the buf.lock file is not found, the bufplugin.NopPluginKeyProvider is returned.
+			if functionOptions.configOverride != "" {
+				// To support remote plugins in the override, we need to resolve the remote
+				// Refs to PluginKeys. A buf.lock file is not required for this operation.
+				// We use the BSR to resolve any remote plugin Refs.
+				pluginKeyProvider = c.pluginKeyProvider
+			} else if bufYAMLFile.FileVersion() == bufconfig.FileVersionV2 {
+				var pluginKeys []bufplugin.PluginKey
+				if bufLockFile, err := bufconfig.GetBufLockFileForPrefix(
+					ctx,
+					bucket,
+					// buf.lock files live next to the buf.yaml
+					".",
+				); err != nil {
+					if !errors.Is(err, fs.ErrNotExist) {
+						return nil, nil, err
+					}
+					// We did not find a buf.lock in our current directory.
+					// Remote plugins are not available.
+					pluginKeys = nil
+				} else {
+					pluginKeys = bufLockFile.RemotePluginKeys()
+				}
+				pluginKeyProvider, err = newStaticPluginKeyProviderForPluginConfigs(
+					pluginConfigs,
+					pluginKeys,
+				)
+				if err != nil {
+					return nil, nil, err
+				}
+			}
 		}
-		return []ImageWithConfig{
+		imageWithConfigs := []ImageWithConfig{
 			newImageWithConfig(
 				image,
 				lintConfig,
 				breakingConfig,
 				pluginConfigs,
 			),
-		}, nil
+		}
+		pluginRunnerProvider := bufcheck.NewLocalRunnerProvider(
+			wasmRuntime,
+			pluginKeyProvider,
+			c.pluginDataProvider,
+		)
+		checkClient, err := bufcheck.NewClient(
+			c.logger,
+			pluginRunnerProvider,
+			bufcheck.ClientWithStderr(c.container.Stderr()),
+		)
+		if err != nil {
+			return nil, nil, err
+		}
+		return imageWithConfigs, checkClient, nil
 	default:
 		// This is a system error.
-		return nil, syserror.Newf("invalid Ref: %T", ref)
+		return nil, nil, syserror.Newf("invalid Ref: %T", ref)
 	}
+	targetImageWithConfigs, err := c.buildTargetImageWithConfigs(ctx, workspace, functionOptions)
+	if err != nil {
+		return nil, nil, err
+	}
+	checkClient, err := c.GetCheckClientForWorkspace(ctx, workspace, wasmRuntime)
+	if err != nil {
+		return nil, nil, err
+	}
+	return targetImageWithConfigs, checkClient, err
 }
 
 func (c *controller) GetImportableImageFileInfos(
@@ -566,7 +654,7 @@ func (c *controller) PutImage(
 		return err
 	}
 	defer func() {
-		retErr = multierr.Append(retErr, writeCloser.Close())
+		retErr = errors.Join(retErr, writeCloser.Close())
 	}()
 	_, err = writeCloser.Write(data)
 	return err
@@ -696,7 +784,31 @@ func (c *controller) PutMessage(
 		return err
 	}
 	_, err = writeCloser.Write(data)
-	return multierr.Append(err, writeCloser.Close())
+	return errors.Join(err, writeCloser.Close())
+}
+
+func (c *controller) GetCheckClientForWorkspace(
+	ctx context.Context,
+	workspace bufworkspace.Workspace,
+	wasmRuntime wasm.Runtime,
+) (_ bufcheck.Client, retErr error) {
+	pluginKeyProvider, err := newStaticPluginKeyProviderForPluginConfigs(
+		workspace.PluginConfigs(),
+		workspace.RemotePluginKeys(),
+	)
+	if err != nil {
+		return nil, err
+	}
+	pluginRunnerProvider := bufcheck.NewLocalRunnerProvider(
+		wasmRuntime,
+		pluginKeyProvider,
+		c.pluginDataProvider,
+	)
+	return bufcheck.NewClient(
+		c.logger,
+		pluginRunnerProvider,
+		bufcheck.ClientWithStderr(c.container.Stderr()),
+	)
 }
 
 func (c *controller) getImage(
@@ -803,7 +915,7 @@ func (c *controller) getWorkspaceForProtoFileRef(
 		return nil, err
 	}
 	defer func() {
-		retErr = multierr.Append(retErr, readBucketCloser.Close())
+		retErr = errors.Join(retErr, readBucketCloser.Close())
 	}()
 	options := []bufworkspace.WorkspaceBucketOption{
 		bufworkspace.WithProtoFileTargetPath(
@@ -843,7 +955,7 @@ func (c *controller) getWorkspaceForSourceRef(
 		return nil, err
 	}
 	defer func() {
-		retErr = multierr.Append(retErr, readBucketCloser.Close())
+		retErr = errors.Join(retErr, readBucketCloser.Close())
 	}()
 	options := []bufworkspace.WorkspaceBucketOption{
 		bufworkspace.WithConfigOverride(
@@ -919,7 +1031,7 @@ func (c *controller) getImageForMessageRef(
 		return nil, err
 	}
 	defer func() {
-		retErr = multierr.Append(retErr, readCloser.Close())
+		retErr = errors.Join(retErr, readCloser.Close())
 	}()
 	data, err := io.ReadAll(readCloser)
 	if err != nil {
@@ -973,8 +1085,8 @@ func (c *controller) getImageForMessageRef(
 	}
 
 	if functionOptions.imageExcludeSourceInfo {
-		for _, fileDescriptorProto := range protoImage.File {
-			fileDescriptorProto.SourceCodeInfo = nil
+		for _, fileDescriptorProto := range protoImage.GetFile() {
+			fileDescriptorProto.ClearSourceCodeInfo()
 		}
 	}
 
@@ -1090,15 +1202,15 @@ func (c *controller) warnUnconfiguredTransitiveImports(
 	if slicesext.Count(workspace.Modules(), bufmodule.Module.IsLocal) == 0 {
 		return nil
 	}
-	// Construct a struct map of all the ModuleFullName strings of the configured buf.yaml
+	// Construct a struct map of all the FullName strings of the configured buf.yaml
 	// Module dependencies, and the local Modules. These are considered OK to depend on
 	// for non-imports in the Image.
-	configuredModuleFullNameStrings, err := slicesext.MapError(
+	configuredFullNameStrings, err := slicesext.MapError(
 		workspace.ConfiguredDepModuleRefs(),
-		func(moduleRef bufmodule.ModuleRef) (string, error) {
-			moduleFullName := moduleRef.ModuleFullName()
+		func(moduleRef bufparse.Ref) (string, error) {
+			moduleFullName := moduleRef.FullName()
 			if moduleFullName == nil {
-				return "", syserror.New("ModuleFullName nil on ModuleRef")
+				return "", syserror.New("FullName nil on ModuleRef")
 			}
 			return moduleFullName.String(), nil
 		},
@@ -1106,23 +1218,23 @@ func (c *controller) warnUnconfiguredTransitiveImports(
 	if err != nil {
 		return err
 	}
-	configuredModuleFullNameStringMap := slicesext.ToStructMap(configuredModuleFullNameStrings)
+	configuredFullNameStringMap := slicesext.ToStructMap(configuredFullNameStrings)
 	for _, localModule := range bufmodule.ModuleSetLocalModules(workspace) {
-		if moduleFullName := localModule.ModuleFullName(); moduleFullName != nil {
-			configuredModuleFullNameStringMap[moduleFullName.String()] = struct{}{}
+		if moduleFullName := localModule.FullName(); moduleFullName != nil {
+			configuredFullNameStringMap[moduleFullName.String()] = struct{}{}
 		}
 	}
 
-	// Construct a map from Image file path -> ModuleFullName string.
+	// Construct a map from Image file path -> FullName string.
 	//
-	// If a given file in the Image did not have a ModuleFullName, it came from a local unnamed Module
+	// If a given file in the Image did not have a FullName, it came from a local unnamed Module
 	// in the Workspace, and we're safe to ignore it with respect to calculating the undeclared
 	// transitive imports.
-	pathToModuleFullNameString := make(map[string]string)
+	pathToFullNameString := make(map[string]string)
 	for _, imageFile := range image.Files() {
 		// If nil, this came from a local unnamed Module in the Workspace, and we're safe to ignore.
-		if moduleFullName := imageFile.ModuleFullName(); moduleFullName != nil {
-			pathToModuleFullNameString[imageFile.Path()] = moduleFullName.String()
+		if moduleFullName := imageFile.FullName(); moduleFullName != nil {
+			pathToFullNameString[imageFile.Path()] = moduleFullName.String()
 		}
 	}
 
@@ -1132,12 +1244,12 @@ func (c *controller) warnUnconfiguredTransitiveImports(
 			continue
 		}
 		for _, importPath := range imageFile.FileDescriptorProto().GetDependency() {
-			moduleFullNameString, ok := pathToModuleFullNameString[importPath]
+			moduleFullNameString, ok := pathToFullNameString[importPath]
 			if !ok {
 				// The import was from a local unnamed Module in the Workspace.
 				continue
 			}
-			if _, ok := configuredModuleFullNameStringMap[moduleFullNameString]; !ok {
+			if _, ok := configuredFullNameStringMap[moduleFullNameString]; !ok {
 				c.logger.Warn(fmt.Sprintf(
 					`File %q imports %q, which is not in your workspace or in the dependencies declared in your buf.yaml, but is found in transitive dependency %q.
 Declare %q in the deps key in your buf.yaml.`,
@@ -1203,7 +1315,7 @@ func bootstrapResolver(
 	if err := unmarshaler.Unmarshal(data, firstProtoImage); err != nil {
 		return nil, err
 	}
-	return protoencoding.NewResolver(firstProtoImage.File...)
+	return protoencoding.NewResolver(firstProtoImage.GetFile()...)
 }
 
 // WE DO NOT FILTER IF WE ALREADY FILTERED ON BUILDING OF A WORKSPACE
@@ -1335,3 +1447,36 @@ func validateFileAnnotationErrorFormat(fileAnnotationErrorFormat string) error {
 	fileAnnotationErrorFormatFlagName := "error-format"
 	return appcmd.NewInvalidArgumentErrorf("--%s: invalid format: %q", fileAnnotationErrorFormatFlagName, fileAnnotationErrorFormat)
 }
+
+// newStaticPluginKeyProvider creates a new PluginKeyProvider for the set of PluginKeys.
+//
+// The PluginKeys come from the buf.lock file. The PluginKeyProvider is static
+// and does not change. PluginConfigs are validated to ensure that all remote
+// PluginConfigs are pinned in the buf.lock file.
+func newStaticPluginKeyProviderForPluginConfigs(
+	pluginConfigs []bufconfig.PluginConfig,
+	pluginKeys []bufplugin.PluginKey,
+) (_ bufplugin.PluginKeyProvider, retErr error) {
+	// Validate that all remote PluginConfigs are present in the buf.lock file.
+	pluginKeysByFullName, err := slicesext.ToUniqueValuesMap(pluginKeys, func(pluginKey bufplugin.PluginKey) string {
+		return pluginKey.FullName().String()
+	})
+	if err != nil {
+		return nil, fmt.Errorf("failed to validate remote PluginKeys: %w", err)
+	}
+	// Remote PluginConfig Refs are any PluginConfigs that have a Ref.
+	remotePluginRefs := slicesext.Filter(
+		slicesext.Map(pluginConfigs, func(pluginConfig bufconfig.PluginConfig) bufparse.Ref {
+			return pluginConfig.Ref()
+		}),
+		func(pluginRef bufparse.Ref) bool {
+			return pluginRef != nil
+		},
+	)
+	for _, remotePluginRef := range remotePluginRefs {
+		if _, ok := pluginKeysByFullName[remotePluginRef.FullName().String()]; !ok {
+			return nil, fmt.Errorf(`remote plugin %q is not in the buf.lock file, use "buf plugin update" to pin remote refs`, remotePluginRef)
+		}
+	}
+	return bufplugin.NewStaticPluginKeyProvider(pluginKeys)
+}
