@@ -1,4 +1,4 @@
-// Copyright 2020-2024 Buf Technologies, Inc.
+// Copyright 2020-2025 Buf Technologies, Inc.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -27,9 +27,11 @@ import (
 	"slices"
 	"strings"
 
+	"github.com/bufbuild/buf/private/bufpkg/bufmodule"
 	"github.com/bufbuild/buf/private/pkg/slicesext"
 	"github.com/bufbuild/protocompile/ast"
 	"go.lsp.dev/protocol"
+	"google.golang.org/protobuf/encoding/protowire"
 )
 
 // symbol represents a named symbol inside of a buflsp.file
@@ -95,10 +97,16 @@ type builtin struct {
 	name string
 }
 
+type fieldTag struct {
+	tag ast.IntValueNode
+	def *symbol
+}
+
 func (*definition) isSymbolKind() {}
 func (*reference) isSymbolKind()  {}
 func (*import_) isSymbolKind()    {}
 func (*builtin) isSymbolKind()    {}
+func (*fieldTag) isSymbolKind()   {}
 
 // Range constructs an LSP protocol code range for this symbol.
 func (s *symbol) Range() protocol.Range {
@@ -115,8 +123,6 @@ func (s *symbol) Definition(ctx context.Context) (*symbol, ast.Node) {
 			return nil, nil
 		}
 
-		kind.file.lock.Lock(ctx)
-		defer kind.file.lock.Unlock(ctx)
 		for _, symbol := range kind.file.symbols {
 			def, ok := symbol.kind.(*definition)
 			if ok && slices.Equal(kind.path, def.path) {
@@ -143,7 +149,7 @@ func (s *symbol) ReferencePath() (path []string, absolute bool) {
 	return
 }
 
-// Resolve attempts to resolve an unresolved reference across fileManager.
+// ResolveCrossFile attempts to resolve an unresolved reference across fileManager.
 func (s *symbol) ResolveCrossFile(ctx context.Context) {
 	switch kind := s.kind.(type) {
 	case *definition:
@@ -170,11 +176,6 @@ func (s *symbol) ResolveCrossFile(ctx context.Context) {
 				return
 			}
 
-			// Fully index the file this reference is in, if different from the current.
-			if s.file != ref.file {
-				ref.file.Refresh(ctx)
-			}
-
 			// Find the definition that contains the type we want.
 			def, node := kind.seeTypeOf.Definition(ctx)
 			if def == nil {
@@ -234,7 +235,7 @@ func (s *symbol) ResolveCrossFile(ctx context.Context) {
 
 			// Done.
 			kind.file = def.file
-			kind.path = append(slicesext.Copy(ref.path), components...)
+			kind.path = append(slices.Clone(ref.path), components...)
 			return
 		}
 
@@ -272,22 +273,11 @@ func (s *symbol) ResolveCrossFile(ctx context.Context) {
 				return
 			}
 
-			// Make a copy of the import table pointer and then drop the lock,
-			// since searching inside of the imports will need to acquire other
-			// fileManager' locks.
-			s.file.lock.Lock(ctx)
+			// Look for a symbol with this exact path in descriptor proto.
 			descriptorProto := s.file.importToFile[descriptorPath]
-			s.file.lock.Unlock(ctx)
-
 			if descriptorProto == nil {
 				return
 			}
-
-			// Look for a symbol with this exact path in descriptor proto.
-
-			descriptorProto.lock.Lock(ctx)
-			defer descriptorProto.lock.Unlock(ctx)
-
 			var fieldSymbol *symbol
 			for _, symbol := range descriptorProto.symbols {
 				if def, ok := symbol.kind.(*definition); ok && slices.Equal(def.path, fieldPath) {
@@ -304,23 +294,35 @@ func (s *symbol) ResolveCrossFile(ctx context.Context) {
 			return
 		}
 
-		// Make a copy of the import table pointer and then drop the lock,
-		// since searching inside of the imports will need to acquire other
-		// fileManager' locks.
-		s.file.lock.Lock(ctx)
-		imports := s.file.importToFile
-		s.file.lock.Unlock(ctx)
-
-		if imports == nil {
+		if s.file.importToFile == nil {
 			// Hopeless. We'll have to try again once we have imports!
 			return
 		}
 
-		for _, imported := range imports {
-			// Remove a leading pkg from components.
+		for _, imported := range s.file.importToFile {
+			// If necessary, refresh the file. Note that this cannot hit
+			// cycles, because fileNode will become non-nil after calling
+			// Refresh but before calling IndexSymbols.
+			if imported.fileNode == nil {
+				imported.Refresh(ctx)
+			}
+
+			// Need to check two paths; components on its own, and components
+			// with s.file's package prepended.
+
+			// First, try removing the imported file's package from components.
 			path, ok := slicesext.TrimPrefix(components, imported.Package())
 			if !ok {
-				continue
+				// If that doesn't work, try appending the importee's package.
+				// This is necessary because protobuf allows for partial package
+				// names to appear in references.
+				path, ok = slicesext.TrimPrefix(
+					slices.Concat(s.file.Package(), components),
+					imported.Package(),
+				)
+				if !ok {
+					continue
+				}
 			}
 
 			if findDeclByPath(imported.fileNode.Decls, path) != nil {
@@ -332,7 +334,48 @@ func (s *symbol) ResolveCrossFile(ctx context.Context) {
 	}
 }
 
-// FormatDocs finds appropriate  documgntation for the given s and constructs a Markdown
+func (s *symbol) LogValue() slog.Value {
+	attrs := []slog.Attr{slog.String("file", s.file.uri.Filename())}
+
+	// pos converts an ast.SourcePos into a slog.Value.
+	pos := func(pos ast.SourcePos) slog.Value {
+		return slog.GroupValue(
+			slog.Int("line", pos.Line),
+			slog.Int("col", pos.Col),
+		)
+	}
+
+	attrs = append(attrs, slog.Any("start", pos(s.info.Start())))
+	attrs = append(attrs, slog.Any("end", pos(s.info.End())))
+
+	switch kind := s.kind.(type) {
+	case *builtin:
+		attrs = append(attrs, slog.String("builtin", kind.name))
+
+	case *import_:
+		if kind.file != nil {
+			attrs = append(attrs, slog.String("imports", kind.file.uri.Filename()))
+		}
+
+	case *definition:
+		attrs = append(attrs, slog.String("defines", strings.Join(kind.path, ".")))
+
+	case *reference:
+		if kind.file != nil {
+			attrs = append(attrs, slog.String("imports", kind.file.uri.Filename()))
+		}
+		if kind.path != nil {
+			attrs = append(attrs, slog.String("references", strings.Join(kind.path, ".")))
+		}
+		if kind.seeTypeOf != nil {
+			attrs = append(attrs, slog.Any("see_type_of", kind.seeTypeOf))
+		}
+	}
+
+	return slog.GroupValue(attrs...)
+}
+
+// FormatDocs finds appropriate documentation for the given s and constructs a Markdown
 // string for showing to the client.
 //
 // Returns the empty string if no docs are available.
@@ -367,10 +410,80 @@ func (s *symbol) FormatDocs(ctx context.Context) string {
 		node = kind.node
 		path = kind.path
 
+	case *import_:
+		return fmt.Sprintf("```proto\n%s\n```", kind.file.text)
+
+	case *fieldTag:
+		var value uint64
+		if v, ok := kind.tag.AsInt64(); ok {
+			value = uint64(v)
+		} else {
+			value, _ = kind.tag.AsUint64()
+		}
+
+		plural := func(i int) string {
+			if i == 1 {
+				return ""
+			}
+			return "s"
+		}
+
+		var ty protowire.Type
+		var packed bool
+		// Only definition symbols are placed into the def field of fieldTag, so
+		// doing an unchecked assertion here is ok.
+		switch def := kind.def.kind.(*definition).node.(type) {
+		case *ast.EnumValueNode:
+			varint := protowire.AppendVarint(nil, value)
+			return fmt.Sprintf(
+				"`0x%x`, `0b%b`\n\nencoded (hex): `%X` (%d byte%s)",
+				value, value, varint, len(varint), plural(len(varint)),
+			)
+		case *ast.MapFieldNode:
+			ty = protowire.BytesType
+		case *ast.GroupNode:
+			ty = protowire.StartGroupType
+		case *ast.FieldNode:
+			switch def.FldType.AsIdentifier() {
+			case "bool", "int32", "int64", "uint32", "uint64", "sint32", "sint64":
+				ty = protowire.VarintType
+				packed = def.Label.Repeated
+			case "fixed32", "sfixed32", "float":
+				ty = protowire.Fixed32Type
+				packed = def.Label.Repeated
+			case "fixed64", "sfixed64", "double":
+				ty = protowire.Fixed64Type
+				packed = def.Label.Repeated
+			default:
+				ty = protowire.BytesType
+			}
+		}
+
+		// Don't use AppendTag because that wants to truncate value to int32.
+		varint := protowire.AppendVarint(nil, value<<3|uint64(ty))
+		doc := fmt.Sprintf(
+			"encoded (hex): `%X` (%d byte%s)",
+			varint, len(varint), plural(len(varint)),
+		)
+
+		if packed {
+			packed := protowire.AppendVarint(nil, value<<3|uint64(protowire.BytesType))
+			return doc + fmt.Sprintf(
+				"\n\npacked (hex): `%X` (%d byte%s)",
+				packed, len(packed), plural(len(varint)),
+			)
+		}
+
+		return doc
+
 	default:
 		return ""
 	}
 
+	if def == nil {
+		return ""
+	}
+
 	pkg := "<empty>"
 	if pkgNode := def.file.packageNode; pkgNode != nil {
 		pkg = string(pkgNode.Name.AsIdentifier())
@@ -413,19 +526,37 @@ func (s *symbol) FormatDocs(ctx context.Context) string {
 		return tooltip.String()
 	}
 
-	if def.file.imageFileInfo != nil {
-		path := strings.Join(path, ".")
+	// Do not show BSR links for local files, because those may contain symbols
+	// that dop not exist remotely.
+	if def.file != nil && !def.file.IsLocal() {
+		// Classify this node by whether or not the BSR has an anchor for it.
+		// Extensions are special: they are grouped under an #extensions anchor.
+		var hasAnchor, isExtension bool
+		switch node := node.(type) {
+		case *ast.MessageNode, *ast.EnumNode, *ast.ServiceNode:
+			hasAnchor = true
+		case *ast.FieldNode:
+			isExtension = node.Extendee != nil
+			hasAnchor = isExtension
+		}
 
-		fmt.Fprintf(
-			&tooltip,
-			"[`%s.%s` on the Buf Schema Registry](https://%s/docs/main:%s#%s.%s)\n\n",
-			pkg,
-			path,
-			def.file.imageFileInfo.ModuleFullName(),
-			pkg,
-			pkg,
-			path,
-		)
+		var bsrHost, bsrAnchor, bsrTooltip string
+		if def.file.IsWKT() {
+			bsrHost = "buf.build/protocolbuffers/wellknowntypes"
+		} else if fileInfo, ok := def.file.objectInfo.(bufmodule.FileInfo); ok {
+			bsrHost = fileInfo.Module().FullName().String()
+		}
+		if hasAnchor {
+			bsrTooltip = pkg + "." + strings.Join(path, ".")
+		} else {
+			bsrTooltip = pkg + "." + strings.Join(path[:len(path)-1], ".")
+		}
+		bsrAnchor = bsrTooltip
+		if isExtension {
+			bsrAnchor = "extensions"
+		}
+
+		fmt.Fprintf(&tooltip, "[`%s` on the Buf Schema Registry](https://%s/docs/main:%s#%s)\n\n", bsrTooltip, bsrHost, pkg, bsrAnchor)
 	}
 
 	// Dump all of the comments into the tooltip. These will be rendered as Markdown automatically
@@ -435,20 +566,12 @@ func (s *symbol) FormatDocs(ctx context.Context) string {
 	var printed bool
 	for _, comments := range allComments {
 		for i := 0; i < comments.Len(); i++ {
-			comment := comments.Index(i).RawText()
-
 			// The compiler does not currently provide comments without their
 			// delimited removed, so we have to do this ourselves.
-			if strings.HasPrefix(comment, "//") {
-				comment = strings.TrimSpace(strings.TrimPrefix(comment, "//"))
-			} else {
-				comment = strings.TrimSpace(strings.TrimSuffix(strings.TrimPrefix(comment, "/*"), "*/"))
-			}
-
+			comment := commentToMarkdown(comments.Index(i).RawText())
 			if comment != "" {
 				printed = true
 			}
-
 			// No need to process Markdown in comment; this Just Works!
 			fmt.Fprintln(&tooltip, comment)
 		}
@@ -461,9 +584,45 @@ func (s *symbol) FormatDocs(ctx context.Context) string {
 	return tooltip.String()
 }
 
+// commentToMarkdown processes comment strings and formats them for markdown display.
+func commentToMarkdown(comment string) string {
+	if strings.HasPrefix(comment, "//") {
+		// NOTE: We do not trim the space here, because indentation is
+		// significant for Markdown code fences, and if every line
+		// starts with a space, Markdown will trim it for us, even off
+		// of code blocks.
+		return strings.TrimPrefix(comment, "//")
+	}
+
+	if strings.HasPrefix(comment, "/**") && !strings.HasPrefix(comment, "/**/") {
+		// NOTE: Doxygen-style comments (/** ... */) to Markdown format
+		// by removing comment delimiters and formatting the content.
+		//
+		// Example:
+		// /**
+		//  * This is a Doxygen comment
+		//  * with multiple lines
+		//  */
+		comment = strings.TrimSuffix(strings.TrimPrefix(comment, "/**"), "*/")
+
+		lines := strings.Split(strings.TrimSpace(comment), "\n")
+		for i, line := range lines {
+			line = strings.TrimSpace(line)
+			line = strings.TrimPrefix(line, "*")
+			lines[i] = line
+		}
+
+		return strings.Join(lines, "\n")
+	}
+
+	// Handle standard multi-line comments (/* ... */)
+	return strings.TrimSuffix(strings.TrimPrefix(comment, "/*"), "*/")
+}
+
 // symbolWalker is an AST walker that generates the symbol table for a file in IndexSymbols().
 type symbolWalker struct {
-	file *file
+	file    *file
+	symbols []*symbol
 
 	// This is the set of *ast.MessageNode, *ast.EnumNode, and *ast.ServiceNode that
 	// we have traversed. They are used for same-file symbol resolution, and for constructing
@@ -534,15 +693,22 @@ func (w *symbolWalker) Walk(node, parent ast.Node) {
 		}
 
 	case *ast.GroupNode:
+		def := w.newDef(node, node.Name)
 		w.newDef(node, node.Name)
+		if node.Tag != nil {
+			w.newTag(node.Tag, def)
+		}
 		// TODO: also do the name of the generated field.
 		for _, decl := range node.Decls {
 			w.Walk(decl, node)
 		}
 
 	case *ast.FieldNode:
-		w.newDef(node, node.Name)
+		def := w.newDef(node, node.Name)
 		w.newRef(node.FldType)
+		if node.Tag != nil {
+			w.newTag(node.Tag, def)
+		}
 		if node.Options != nil {
 			for _, option := range node.Options.Options {
 				w.Walk(option, node)
@@ -550,9 +716,12 @@ func (w *symbolWalker) Walk(node, parent ast.Node) {
 		}
 
 	case *ast.MapFieldNode:
-		w.newDef(node, node.Name)
+		def := w.newDef(node, node.Name)
 		w.newRef(node.MapType.KeyType)
 		w.newRef(node.MapType.ValueType)
+		if node.Tag != nil {
+			w.newTag(node.Tag, def)
+		}
 		if node.Options != nil {
 			for _, option := range node.Options.Options {
 				w.Walk(option, node)
@@ -576,7 +745,10 @@ func (w *symbolWalker) Walk(node, parent ast.Node) {
 		}
 
 	case *ast.EnumValueNode:
-		w.newDef(node, node.Name)
+		def := w.newDef(node, node.Name)
+		if node.Number != nil {
+			w.newTag(node.Number, def)
+		}
 		if node.Options != nil {
 			for _, option := range node.Options.Options {
 				w.Walk(option, node)
@@ -613,7 +785,7 @@ func (w *symbolWalker) Walk(node, parent ast.Node) {
 				}
 			} else {
 				// This depends on the type of the previous symbol.
-				prev := w.file.symbols[len(w.file.symbols)-1]
+				prev := w.symbols[len(w.symbols)-1]
 				next = w.newSymbol(part.Name)
 				next.kind = &reference{seeTypeOf: prev}
 			}
@@ -634,7 +806,7 @@ func (w *symbolWalker) newSymbol(name ast.Node) *symbol {
 		info: w.file.fileNode.NodeInfo(name),
 	}
 
-	w.file.symbols = append(w.file.symbols, symbol)
+	w.symbols = append(w.symbols, symbol)
 	return symbol
 }
 
@@ -650,6 +822,15 @@ func (w *symbolWalker) newDef(node ast.Node, name *ast.IdentNode) *symbol {
 	return symbol
 }
 
+// newTag creates a new symbol for a field tag, and adds it to the running list.
+//
+// Returns a new symbol for that tag.
+func (w *symbolWalker) newTag(tag ast.IntValueNode, def *symbol) *symbol {
+	symbol := w.newSymbol(tag)
+	symbol.kind = &fieldTag{tag, def}
+	return symbol
+}
+
 // newDef creates a new symbol for a name reference, and adds it to the running list.
 //
 // newRef performs same-file Protobuf name resolution. It searches for a partial package
