@@ -1,4 +1,4 @@
-// Copyright 2020-2024 Buf Technologies, Inc.
+// Copyright 2020-2025 Buf Technologies, Inc.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -277,7 +277,7 @@ func checkConstraintsForField(
 	case boolRulesFieldNumber:
 		// Bool rules only have `const` and does not need validating.
 	case stringRulesFieldNumber:
-		return checkStringRules(adder, fieldConstraints.GetString_())
+		return checkStringRules(adder, fieldConstraints.GetString())
 	case bytesRulesFieldNumber:
 		return checkBytesRules(adder, fieldConstraints.GetBytes())
 	case enumRulesFieldNumber:
@@ -826,7 +826,7 @@ func checkExampleValues(
 	}
 	// The shape of field path in a protovalidate.Violation depends on the type of the field descriptor.
 	violationFilterFunc := func(violation *validate.Violation) bool {
-		return violation.GetFieldPath() == string(fieldDescriptor.Name())
+		return protovalidate.FieldPathString(violation.GetField()) == string(fieldDescriptor.Name())
 	}
 	switch {
 	case fieldDescriptor.IsList():
@@ -834,7 +834,7 @@ func checkExampleValues(
 		violationFilterFunc = func(violation *validate.Violation) bool {
 			prefix := fieldDescriptor.Name() + "["
 			suffix := "]"
-			fieldPath := violation.GetFieldPath()
+			fieldPath := protovalidate.FieldPathString(violation.GetField())
 			return strings.HasPrefix(fieldPath, string(prefix)) && strings.HasSuffix(fieldPath, suffix)
 		}
 	case parentMapFieldDescriptor != nil && fieldDescriptor.Name() == "key":
@@ -842,7 +842,7 @@ func checkExampleValues(
 		violationFilterFunc = func(violation *validate.Violation) bool {
 			prefix := parentMapFieldDescriptor.Name() + "["
 			suffix := "]"
-			fieldPath := violation.GetFieldPath()
+			fieldPath := protovalidate.FieldPathString(violation.GetField())
 			return strings.HasPrefix(fieldPath, string(prefix)) && strings.HasSuffix(fieldPath, suffix) && violation.GetForKey()
 		}
 	case parentMapFieldDescriptor != nil && fieldDescriptor.Name() == "value":
@@ -850,7 +850,7 @@ func checkExampleValues(
 		violationFilterFunc = func(violation *validate.Violation) bool {
 			prefix := parentMapFieldDescriptor.Name() + "["
 			suffix := "]"
-			fieldPath := violation.GetFieldPath()
+			fieldPath := protovalidate.FieldPathString(violation.GetField())
 			return strings.HasPrefix(fieldPath, string(prefix)) && strings.HasSuffix(fieldPath, suffix) && !violation.GetForKey()
 		}
 	}
@@ -889,7 +889,7 @@ func checkExampleValues(
 					exampleValue,
 				)
 			default:
-				return syserror.Newf("expected key or value as sythetic field name for map entry's field name, got %q", fieldDescriptor.Name())
+				return syserror.Newf("expected key or value as synthetic field name for map entry's field name, got %q", fieldDescriptor.Name())
 			}
 			messageToValidate.Set(parentMapFieldDescriptor, protoreflect.ValueOfMap(mapEntry))
 		case fieldDescriptor.Enum() != nil:
@@ -1026,8 +1026,8 @@ func checkExampleValues(
 		validationErr := &protovalidate.ValidationError{}
 		if errors.As(err, &validationErr) {
 			for _, violation := range validationErr.Violations {
-				if violationFilterFunc(violation) {
-					adder.addForPathf(append(pathToExampleValues, int32(exampleValueIndex)), `"%v" is an example value but does not satisfy rule %q: %s`, exampleValue.Interface(), violation.GetConstraintId(), violation.GetMessage())
+				if violationFilterFunc(violation.Proto) {
+					adder.addForPathf(append(pathToExampleValues, int32(exampleValueIndex)), `"%v" is an example value but does not satisfy rule %q: %s`, exampleValue.Interface(), violation.Proto.GetConstraintId(), violation.Proto.GetMessage())
 				}
 			}
 			continue
