@@ -1,4 +1,4 @@
-// Copyright 2020-2024 Buf Technologies, Inc.
+// Copyright 2020-2025 Buf Technologies, Inc.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -27,7 +27,7 @@ import (
 	"github.com/bufbuild/buf/private/bufpkg/bufcheck"
 	"github.com/bufbuild/buf/private/bufpkg/bufimage"
 	"github.com/bufbuild/buf/private/bufpkg/bufmodule"
-	"github.com/bufbuild/buf/private/pkg/command"
+	"github.com/bufbuild/buf/private/bufpkg/bufplugin"
 	"github.com/bufbuild/buf/private/pkg/slogtestext"
 	"github.com/bufbuild/buf/private/pkg/storage/storageos"
 	"github.com/bufbuild/buf/private/pkg/wasm"
@@ -172,17 +172,6 @@ func TestRunImportNoPublic(t *testing.T) {
 	)
 }
 
-func TestRunImportNoWeak(t *testing.T) {
-	t.Parallel()
-	testLint(
-		t,
-		"import_no_weak",
-		bufanalysistesting.NewFileAnnotation(t, "a.proto", 6, 1, 6, 30, "IMPORT_NO_WEAK"),
-		bufanalysistesting.NewFileAnnotation(t, "a.proto", 7, 1, 7, 29, "IMPORT_NO_WEAK"),
-		bufanalysistesting.NewFileAnnotation(t, "one/one.proto", 6, 1, 6, 30, "IMPORT_NO_WEAK"),
-	)
-}
-
 func TestRunImportUsed(t *testing.T) {
 	t.Parallel()
 	testLint(
@@ -1270,6 +1259,15 @@ func TestRunLintCustomWasmPlugins(t *testing.T) {
 	)
 }
 
+func TestRunLintEditionsGoFeatures(t *testing.T) {
+	t.Parallel()
+	testLint(
+		t,
+		"editions_go_features",
+		bufanalysistesting.NewFileAnnotationNoLocation(t, "a.proto", "PACKAGE_DEFINED"),
+	)
+}
+
 func testLint(
 	t *testing.T,
 	relDirPath string,
@@ -1319,6 +1317,7 @@ func testLintWithOptions(
 		bufmodule.NopGraphProvider,
 		bufmodule.NopModuleDataProvider,
 		bufmodule.NopCommitProvider,
+		bufplugin.NopPluginKeyProvider,
 	).GetWorkspaceForBucket(
 		ctx,
 		readWriteBucket,
@@ -1356,7 +1355,11 @@ func testLintWithOptions(
 	})
 	client, err := bufcheck.NewClient(
 		logger,
-		bufcheck.NewRunnerProvider(command.NewRunner(), wasmRuntime),
+		bufcheck.NewLocalRunnerProvider(
+			wasmRuntime,
+			bufplugin.NopPluginKeyProvider,
+			bufplugin.NopPluginDataProvider,
+		),
 	)
 	require.NoError(t, err)
 	err = client.Lint(
