@@ -1,4 +1,4 @@
-// Copyright 2020-2024 Buf Technologies, Inc.
+// Copyright 2020-2025 Buf Technologies, Inc.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -22,7 +22,6 @@ import (
 	"strings"
 
 	"github.com/bufbuild/buf/private/pkg/app"
-	"github.com/bufbuild/buf/private/pkg/command"
 	"github.com/bufbuild/buf/private/pkg/git"
 )
 
@@ -31,7 +30,7 @@ func main() {
 }
 
 func run(ctx context.Context, container app.Container) error {
-	files, err := git.NewLister(command.NewRunner()).ListFilesAndUnstagedFiles(
+	files, err := git.NewLister().ListFilesAndUnstagedFiles(
 		ctx,
 		container,
 		git.ListFilesAndUnstagedFilesOptions{},
